@@ -13,20 +13,110 @@
 ##############################################################################
 """Adapter management
 """
+import itertools
 import weakref
 
+from zope.interface import Interface
 from zope.interface import implementer
 from zope.interface import providedBy
-from zope.interface import Interface
 from zope.interface import ro
+from zope.interface._compat import _normalize_name
+from zope.interface._compat import _use_c_impl
 from zope.interface.interfaces import IAdapterRegistry
 
-from zope.interface._compat import _normalize_name
-from zope.interface._compat import STRING_TYPES
 
-_BLANK = u''
+__all__ = [
+    'AdapterRegistry',
+    'VerifyingAdapterRegistry',
+]
 
-class BaseAdapterRegistry(object):
+# In the CPython implementation,
+# ``tuple`` and ``list`` cooperate so that ``tuple([some list])``
+# directly allocates and iterates at the C level without using a
+# Python iterator. That's not the case for
+# ``tuple(generator_expression)`` or ``tuple(map(func, it))``.
+##
+# 3.8
+# ``tuple([t for t in range(10)])``      -> 610ns
+# ``tuple(t for t in range(10))``        -> 696ns
+# ``tuple(map(lambda t: t, range(10)))`` -> 881ns
+##
+# 2.7
+# ``tuple([t fon t in range(10)])``      -> 625ns
+# ``tuple(t for t in range(10))``        -> 665ns
+# ``tuple(map(lambda t: t, range(10)))`` -> 958ns
+#
+# All three have substantial variance.
+##
+# On PyPy, this is also the best option.
+##
+# PyPy 2.7.18-7.3.3
+# ``tuple([t fon t in range(10)])``      -> 128ns
+# ``tuple(t for t in range(10))``        -> 175ns
+# ``tuple(map(lambda t: t, range(10)))`` -> 153ns
+##
+# PyPy 3.7.9 7.3.3-beta
+# ``tuple([t fon t in range(10)])``      ->  82ns
+# ``tuple(t for t in range(10))``        -> 177ns
+# ``tuple(map(lambda t: t, range(10)))`` -> 168ns
+#
+
+class BaseAdapterRegistry:
+    """
+    A basic implementation of the data storage and algorithms required
+    for a :class:`zope.interface.interfaces.IAdapterRegistry`.
+
+    Subclasses can set the following attributes to control how the data
+    is stored; in particular, these hooks can be helpful for ZODB
+    persistence. They can be class attributes that are the named (or similar) type, or
+    they can be methods that act as a constructor for an object that behaves
+    like the types defined here; this object will not assume that they are type
+    objects, but subclasses are free to do so:
+
+    _sequenceType = list
+      This is the type used for our two mutable top-level "byorder" sequences.
+      Must support mutation operations like ``append()`` and ``del seq[index]``.
+      These are usually small (< 10). Although at least one of them is
+      accessed when performing lookups or queries on this object, the other
+      is untouched. In many common scenarios, both are only required when
+      mutating registrations and subscriptions (like what
+      :meth:`zope.interface.interfaces.IComponents.registerUtility` does).
+      This use pattern makes it an ideal candidate to be a
+      :class:`~persistent.list.PersistentList`.
+    _leafSequenceType = tuple
+      This is the type used for the leaf sequences of subscribers.
+      It could be set to a ``PersistentList`` to avoid many unnecessary data
+      loads when subscribers aren't being used. Mutation operations are directed
+      through :meth:`_addValueToLeaf` and :meth:`_removeValueFromLeaf`; if you use
+      a mutable type, you'll need to override those.
+    _mappingType = dict
+      This is the mutable mapping type used for the keyed mappings.
+      A :class:`~persistent.mapping.PersistentMapping`
+      could be used to help reduce the number of data loads when the registry is large
+      and parts of it are rarely used. Further reductions in data loads can come from
+      using a :class:`~BTrees.OOBTree.OOBTree`, but care is required
+      to be sure that all required/provided
+      values are fully ordered (e.g., no required or provided values that are classes
+      can be used).
+    _providedType = dict
+      This is the mutable mapping type used for the ``_provided`` mapping.
+      This is separate from the generic mapping type because the values
+      are always integers, so one might choose to use a more optimized data
+      structure such as a :class:`~BTrees.OIBTree.OIBTree`.
+      The same caveats regarding key types
+      apply as for ``_mappingType``.
+
+    It is possible to also set these on an instance, but because of the need to
+    potentially also override :meth:`_addValueToLeaf` and :meth:`_removeValueFromLeaf`,
+    this may be less useful in a persistent scenario; using a subclass is recommended.
+
+    .. versionchanged:: 5.3.0
+        Add support for customizing the way internal data
+        structures are created.
+    .. versionchanged:: 5.3.0
+        Add methods :meth:`rebuild`, :meth:`allRegistrations`
+        and :meth:`allSubscriptions`.
+    """
 
     # List of methods copied from lookup sub-objects:
     _delegated = ('lookup', 'queryMultiAdapter', 'lookup1', 'queryAdapter',
@@ -52,15 +142,15 @@ class BaseAdapterRegistry(object):
         # but for order == 2 (that is, self._adapters[2]), we have:
         #   {r1 -> {r2 -> {provided -> {name -> value}}}}
         #
-        self._adapters = []
+        self._adapters = self._sequenceType()
 
         # {order -> {required -> {provided -> {name -> [value]}}}}
         # where the remarks about adapters above apply
-        self._subscribers = []
+        self._subscribers = self._sequenceType()
 
         # Set, with a reference count, keeping track of the interfaces
         # for which we have provided components:
-        self._provided = {}
+        self._provided = self._providedType()
 
         # Create ``_v_lookup`` object to perform lookup.  We make this a
         # separate object to to make it easier to implement just the
@@ -85,6 +175,12 @@ class BaseAdapterRegistry(object):
         self.__bases__ = bases
 
     def _setBases(self, bases):
+        """
+        If subclasses need to track when ``__bases__`` changes, they
+        can override this method.
+
+        Subclasses must still call this method.
+        """
         self.__dict__['__bases__'] = bases
         self.ro = ro.ro(self)
         self.changed(self)
@@ -98,30 +194,92 @@ class BaseAdapterRegistry(object):
         for name in self._delegated:
             self.__dict__[name] = getattr(self._v_lookup, name)
 
+    # Hooks for subclasses to define the types of objects used in
+    # our data structures.
+    # These have to be documented in the docstring, instead of local
+    # comments, because Sphinx autodoc ignores the comment and just writes
+    # "alias of list"
+    _sequenceType = list
+    _leafSequenceType = tuple
+    _mappingType = dict
+    _providedType = dict
+
+    def _addValueToLeaf(self, existing_leaf_sequence, new_item):
+        """
+        Add the value *new_item* to the *existing_leaf_sequence*, which may
+        be ``None``.
+
+        Subclasses that redefine `_leafSequenceType` should override this method.
+
+        :param existing_leaf_sequence:
+            If *existing_leaf_sequence* is not *None*, it will be an instance
+            of `_leafSequenceType`. (Unless the object has been unpickled
+            from an old pickle and the class definition has changed, in which case
+            it may be an instance of a previous definition, commonly a `tuple`.)
+
+        :return:
+           This method returns the new value to be stored. It may mutate the
+           sequence in place if it was not ``None`` and the type is mutable, but
+           it must also return it.
+
+        .. versionadded:: 5.3.0
+        """
+        if existing_leaf_sequence is None:
+            return (new_item,)
+        return existing_leaf_sequence + (new_item,)
+
+    def _removeValueFromLeaf(self, existing_leaf_sequence, to_remove):
+        """
+        Remove the item *to_remove* from the (non-``None``, non-empty)
+        *existing_leaf_sequence* and return the mutated sequence.
+
+        If there is more than one item that is equal to *to_remove*
+        they must all be removed.
+
+        Subclasses that redefine `_leafSequenceType` should override
+        this method. Note that they can call this method to help
+        in their implementation; this implementation will always
+        return a new tuple constructed by iterating across
+        the *existing_leaf_sequence* and omitting items equal to *to_remove*.
+
+        :param existing_leaf_sequence:
+           As for `_addValueToLeaf`, probably an instance of
+           `_leafSequenceType` but possibly an older type; never `None`.
+        :return:
+           A version of *existing_leaf_sequence* with all items equal to
+           *to_remove* removed. Must not return `None`. However,
+           returning an empty
+           object, even of another type such as the empty tuple, ``()`` is
+           explicitly allowed; such an object will never be stored.
+
+        .. versionadded:: 5.3.0
+        """
+        return tuple([v for v in existing_leaf_sequence if v != to_remove])
+
     def changed(self, originally_changed):
         self._generation += 1
         self._v_lookup.changed(originally_changed)
 
     def register(self, required, provided, name, value):
-        if not isinstance(name, STRING_TYPES):
+        if not isinstance(name, str):
             raise ValueError('name is not a string')
         if value is None:
             self.unregister(required, provided, name, value)
             return
 
-        required = tuple(map(_convert_None_to_Interface, required))
+        required = tuple([_convert_None_to_Interface(r) for r in required])
         name = _normalize_name(name)
         order = len(required)
         byorder = self._adapters
         while len(byorder) <= order:
-            byorder.append({})
+            byorder.append(self._mappingType())
         components = byorder[order]
         key = required + (provided,)
 
         for k in key:
             d = components.get(k)
             if d is None:
-                d = {}
+                d = self._mappingType()
                 components[k] = d
             components = d
 
@@ -137,11 +295,14 @@ class BaseAdapterRegistry(object):
 
         self.changed(self)
 
-    def registered(self, required, provided, name=_BLANK):
-        required = tuple(map(_convert_None_to_Interface, required))
-        name = _normalize_name(name)
+    def _find_leaf(self, byorder, required, provided, name):
+        # Find the leaf value, if any, in the *byorder* list
+        # for the interface sequence *required* and the interface
+        # *provided*, given the already normalized *name*.
+        #
+        # If no such leaf value exists, returns ``None``
+        required = tuple([_convert_None_to_Interface(r) for r in required])
         order = len(required)
-        byorder = self._adapters
         if len(byorder) <= order:
             return None
 
@@ -156,8 +317,57 @@ class BaseAdapterRegistry(object):
 
         return components.get(name)
 
+    def registered(self, required, provided, name=''):
+        return self._find_leaf(
+            self._adapters,
+            required,
+            provided,
+            _normalize_name(name)
+        )
+
+    @classmethod
+    def _allKeys(cls, components, i, parent_k=()):
+        if i == 0:
+            for k, v in components.items():
+                yield parent_k + (k,), v
+        else:
+            for k, v in components.items():
+                new_parent_k = parent_k + (k,)
+                yield from cls._allKeys(v, i - 1, new_parent_k)
+
+    def _all_entries(self, byorder):
+        # Recurse through the mapping levels of the `byorder` sequence,
+        # reconstructing a flattened sequence of ``(required, provided, name, value)``
+        # tuples that can be used to reconstruct the sequence with the appropriate
+        # registration methods.
+        #
+        # Locally reference the `byorder` data; it might be replaced while
+        # this method is running (see ``rebuild``).
+        for i, components in enumerate(byorder):
+            # We will have *i* levels of dictionaries to go before
+            # we get to the leaf.
+            for key, value in self._allKeys(components, i + 1):
+                assert len(key) == i + 2
+                required = key[:i]
+                provided = key[-2]
+                name = key[-1]
+                yield (required, provided, name, value)
+
+    def allRegistrations(self):
+        """
+        Yields tuples ``(required, provided, name, value)`` for all
+        the registrations that this object holds.
+
+        These tuples could be passed as the arguments to the
+        :meth:`register` method on another adapter registry to
+        duplicate the registrations this object holds.
+
+        .. versionadded:: 5.3.0
+        """
+        yield from self._all_entries(self._adapters)
+
     def unregister(self, required, provided, name, value=None):
-        required = tuple(map(_convert_None_to_Interface, required))
+        required = tuple([_convert_None_to_Interface(r) for r in required])
         order = len(required)
         byorder = self._adapters
         if order >= len(byorder):
@@ -205,23 +415,23 @@ class BaseAdapterRegistry(object):
         self.changed(self)
 
     def subscribe(self, required, provided, value):
-        required = tuple(map(_convert_None_to_Interface, required))
-        name = _BLANK
+        required = tuple([_convert_None_to_Interface(r) for r in required])
+        name = ''
         order = len(required)
         byorder = self._subscribers
         while len(byorder) <= order:
-            byorder.append({})
+            byorder.append(self._mappingType())
         components = byorder[order]
         key = required + (provided,)
 
         for k in key:
             d = components.get(k)
             if d is None:
-                d = {}
+                d = self._mappingType()
                 components[k] = d
             components = d
 
-        components[name] = components.get(name, ()) + (value, )
+        components[name] = self._addValueToLeaf(components.get(name), value)
 
         if provided is not None:
             n = self._provided.get(provided, 0) + 1
@@ -231,8 +441,32 @@ class BaseAdapterRegistry(object):
 
         self.changed(self)
 
+    def subscribed(self, required, provided, subscriber):
+        subscribers = self._find_leaf(
+            self._subscribers,
+            required,
+            provided,
+            ''
+        ) or ()
+        return subscriber if subscriber in subscribers else None
+
+    def allSubscriptions(self):
+        """
+        Yields tuples ``(required, provided, value)`` for all the
+        subscribers that this object holds.
+
+        These tuples could be passed as the arguments to the
+        :meth:`subscribe` method on another adapter registry to
+        duplicate the registrations this object holds.
+
+        .. versionadded:: 5.3.0
+        """
+        for required, provided, _name, value in self._all_entries(self._subscribers):
+            for v in value:
+                yield (required, provided, v)
+
     def unsubscribe(self, required, provided, value=None):
-        required = tuple(map(_convert_None_to_Interface, required))
+        required = tuple([_convert_None_to_Interface(r) for r in required])
         order = len(required)
         byorder = self._subscribers
         if order >= len(byorder):
@@ -249,29 +483,38 @@ class BaseAdapterRegistry(object):
             lookups.append((components, k))
             components = d
 
-        old = components.get(_BLANK)
+        old = components.get('')
         if not old:
             # this is belt-and-suspenders against the failure of cleanup below
-            return  #pragma NO COVERAGE 
-
+            return  # pragma: no cover
+        len_old = len(old)
         if value is None:
+            # Removing everything; note that the type of ``new`` won't
+            # necessarily match the ``_leafSequenceType``, but that's
+            # OK because we're about to delete the entire entry
+            # anyway.
             new = ()
         else:
-            new = tuple([v for v in old if v is not value])
+            new = self._removeValueFromLeaf(old, value)
+        # ``new`` may be the same object as ``old``, just mutated in place,
+        # so we cannot compare it to ``old`` to check for changes. Remove
+        # our reference to it now to avoid trying to do so below.
+        del old
 
-        if new == old:
+        if len(new) == len_old:
+            # No changes, so nothing could have been removed.
             return
 
         if new:
-            components[_BLANK] = new
+            components[''] = new
         else:
-            # Instead of setting components[_BLANK] = new, we clean out
+            # Instead of setting components[u''] = new, we clean out
             # empty containers, since we don't want our keys to
             # reference global objects (interfaces) unnecessarily.  This
             # is often a problem when an interface is slated for
             # removal; a hold-over entry in the registry can make it
             # difficult to remove such interfaces.
-            del components[_BLANK]
+            del components['']
             for comp, k in reversed(lookups):
                 d = comp[k]
                 if d:
@@ -282,23 +525,80 @@ class BaseAdapterRegistry(object):
                 del byorder[-1]
 
         if provided is not None:
-            n = self._provided[provided] + len(new) - len(old)
+            n = self._provided[provided] + len(new) - len_old
             if n == 0:
                 del self._provided[provided]
                 self._v_lookup.remove_extendor(provided)
+            else:
+                self._provided[provided] = n
 
         self.changed(self)
 
+    def rebuild(self):
+        """
+        Rebuild (and replace) all the internal data structures of this
+        object.
+
+        This is useful, especially for persistent implementations, if
+        you suspect an issue with reference counts keeping interfaces
+        alive even though they are no longer used.
+
+        It is also useful if you or a subclass change the data types
+        (``_mappingType`` and friends) that are to be used.
+
+        This method replaces all internal data structures with new objects;
+        it specifically does not re-use any storage.
+
+        .. versionadded:: 5.3.0
+        """
+
+        # Grab the iterators, we're about to discard their data.
+        registrations = self.allRegistrations()
+        subscriptions = self.allSubscriptions()
+
+        def buffer(it):
+            # The generator doesn't actually start running until we
+            # ask for its next(), by which time the attributes will change
+            # unless we do so before calling __init__.
+            try:
+                first = next(it)
+            except StopIteration:
+                return iter(())
+
+            return itertools.chain((first,), it)
+
+        registrations = buffer(registrations)
+        subscriptions = buffer(subscriptions)
+
+
+        # Replace the base data structures as well as _v_lookup.
+        self.__init__(self.__bases__)
+        # Re-register everything previously registered and subscribed.
+        #
+        # XXX: This is going to call ``self.changed()`` a lot, all of
+        # which is unnecessary (because ``self.__init__`` just
+        # re-created those dependent objects and also called
+        # ``self.changed()``). Is this a bottleneck that needs fixed?
+        # (We could do ``self.changed = lambda _: None`` before
+        # beginning and remove it after to disable the presumably expensive
+        # part of passing that notification to the change of objects.)
+        for args in registrations:
+            self.register(*args)
+        for args in subscriptions:
+            self.subscribe(*args)
+
     # XXX hack to fake out twisted's use of a private api.  We need to get them
-    # to use the new registed method.
-    def get(self, _): #pragma NO COVER
+    # to use the new registered method.
+    def get(self, _): # pragma: no cover
         class XXXTwistedFakeOut:
             selfImplied = {}
         return XXXTwistedFakeOut
 
 
 _not_in_mapping = object()
-class LookupBaseFallback(object):
+
+@_use_c_impl
+class LookupBase:
 
     def __init__(self):
         self._cache = {}
@@ -323,8 +623,8 @@ class LookupBaseFallback(object):
             cache = c
         return cache
 
-    def lookup(self, required, provided, name=_BLANK, default=None):
-        if not isinstance(name, STRING_TYPES):
+    def lookup(self, required, provided, name='', default=None):
+        if not isinstance(name, str):
             raise ValueError('name is not a string')
         cache = self._getcache(provided, name)
         required = tuple(required)
@@ -345,8 +645,8 @@ class LookupBaseFallback(object):
 
         return result
 
-    def lookup1(self, required, provided, name=_BLANK, default=None):
-        if not isinstance(name, STRING_TYPES):
+    def lookup1(self, required, provided, name='', default=None):
+        if not isinstance(name, str):
             raise ValueError('name is not a string')
         cache = self._getcache(provided, name)
         result = cache.get(required, _not_in_mapping)
@@ -358,11 +658,11 @@ class LookupBaseFallback(object):
 
         return result
 
-    def queryAdapter(self, object, provided, name=_BLANK, default=None):
+    def queryAdapter(self, object, provided, name='', default=None):
         return self.adapter_hook(provided, object, name, default)
 
-    def adapter_hook(self, provided, object, name=_BLANK, default=None):
-        if not isinstance(name, STRING_TYPES):
+    def adapter_hook(self, provided, object, name='', default=None):
+        if not isinstance(name, str):
             raise ValueError('name is not a string')
         required = providedBy(object)
         cache = self._getcache(provided, name)
@@ -371,6 +671,8 @@ class LookupBaseFallback(object):
             factory = self.lookup((required, ), provided, name)
 
         if factory is not None:
+            if isinstance(object, super):
+                object = object.__self__
             result = factory(object)
             if result is not None:
                 return result
@@ -406,18 +708,12 @@ class LookupBaseFallback(object):
 
         return result
 
-LookupBasePy = LookupBaseFallback # BBB
 
-try:
-    from zope.interface._zope_interface_coptimizations import LookupBase
-except ImportError: #pragma NO COVER
-    LookupBase = LookupBaseFallback
-
-
-class VerifyingBaseFallback(LookupBaseFallback):
+@_use_c_impl
+class VerifyingBase(LookupBaseFallback):
     # Mixin for lookups against registries which "chain" upwards, and
     # whose lookups invalidate their own caches whenever a parent registry
-    # bumps its own '_generation' counter.  E.g., used by 
+    # bumps its own '_generation' counter.  E.g., used by
     # zope.component.persistentregistry
 
     def changed(self, originally_changed):
@@ -442,24 +738,17 @@ class VerifyingBaseFallback(LookupBaseFallback):
         self._verify()
         return LookupBaseFallback.subscriptions(self, required, provided)
 
-VerifyingBasePy = VerifyingBaseFallback #BBB
 
-try:
-    from zope.interface._zope_interface_coptimizations import VerifyingBase
-except ImportError: #pragma NO COVER
-    VerifyingBase = VerifyingBaseFallback
-
-
-class AdapterLookupBase(object):
+class AdapterLookupBase:
 
     def __init__(self, registry):
         self._registry = registry
         self._required = {}
         self.init_extendors()
-        super(AdapterLookupBase, self).__init__()
+        super().__init__()
 
     def changed(self, ignored=None):
-        super(AdapterLookupBase, self).changed(None)
+        super().changed(None)
         for r in self._required.keys():
             r = r()
             if r is not None:
@@ -521,7 +810,7 @@ class AdapterLookupBase(object):
                 r.subscribe(self)
                 _refs[ref] = 1
 
-    def _uncached_lookup(self, required, provided, name=_BLANK):
+    def _uncached_lookup(self, required, provided, name=''):
         required = tuple(required)
         result = None
         order = len(required)
@@ -544,12 +833,12 @@ class AdapterLookupBase(object):
 
         return result
 
-    def queryMultiAdapter(self, objects, provided, name=_BLANK, default=None):
-        factory = self.lookup(map(providedBy, objects), provided, name)
+    def queryMultiAdapter(self, objects, provided, name='', default=None):
+        factory = self.lookup([providedBy(o) for o in objects], provided, name)
         if factory is None:
             return default
 
-        result = factory(*objects)
+        result = factory(*[o.__self__ if isinstance(o, super) else o for o in objects])
         if result is None:
             return default
 
@@ -592,7 +881,7 @@ class AdapterLookupBase(object):
                 if extendors is None:
                     continue
 
-            _subscriptions(byorder[order], required, extendors, _BLANK,
+            _subscriptions(byorder[order], required, extendors, '',
                            result, 0, order)
 
         self._subscribe(*required)
@@ -600,7 +889,7 @@ class AdapterLookupBase(object):
         return result
 
     def subscribers(self, objects, provided):
-        subscriptions = self.subscriptions(map(providedBy, objects), provided)
+        subscriptions = self.subscriptions([providedBy(o) for o in objects], provided)
         if provided is None:
             result = ()
             for subscription in subscriptions:
@@ -618,15 +907,19 @@ class AdapterLookup(AdapterLookupBase, LookupBase):
 
 @implementer(IAdapterRegistry)
 class AdapterRegistry(BaseAdapterRegistry):
+    """
+    A full implementation of ``IAdapterRegistry`` that adds support for
+    sub-registries.
+    """
 
     LookupClass = AdapterLookup
 
     def __init__(self, bases=()):
         # AdapterRegisties are invalidating registries, so
-        # we need to keep track of out invalidating subregistries.
+        # we need to keep track of our invalidating subregistries.
         self._v_subregistries = weakref.WeakKeyDictionary()
 
-        super(AdapterRegistry, self).__init__(bases)
+        super().__init__(bases)
 
     def _addSubregistry(self, r):
         self._v_subregistries[r] = 1
@@ -644,10 +937,10 @@ class AdapterRegistry(BaseAdapterRegistry):
             if r not in old:
                 r._addSubregistry(self)
 
-        super(AdapterRegistry, self)._setBases(bases)
+        super()._setBases(bases)
 
     def changed(self, originally_changed):
-        super(AdapterRegistry, self).changed(originally_changed)
+        super().changed(originally_changed)
 
         for sub in self._v_subregistries.keys():
             sub.changed(originally_changed)
@@ -658,6 +951,9 @@ class VerifyingAdapterLookup(AdapterLookupBase, VerifyingBase):
 
 @implementer(IAdapterRegistry)
 class VerifyingAdapterRegistry(BaseAdapterRegistry):
+    """
+    The most commonly-used adapter registry.
+    """
 
     LookupClass = VerifyingAdapterLookup
 
@@ -668,16 +964,21 @@ def _convert_None_to_Interface(x):
         return x
 
 def _lookup(components, specs, provided, name, i, l):
+    # this function is called very often.
+    # The components.get in loops is executed 100 of 1000s times.
+    # by loading get into a local variable the bytecode
+    # "LOAD_FAST 0 (components)" in the loop can be eliminated.
+    components_get = components.get
     if i < l:
         for spec in specs[i].__sro__:
-            comps = components.get(spec)
+            comps = components_get(spec)
             if comps:
                 r = _lookup(comps, specs, provided, name, i+1, l)
                 if r is not None:
                     return r
     else:
         for iface in provided:
-            comps = components.get(iface)
+            comps = components_get(iface)
             if comps:
                 r = comps.get(name)
                 if r is not None:
@@ -686,26 +987,28 @@ def _lookup(components, specs, provided, name, i, l):
     return None
 
 def _lookupAll(components, specs, provided, result, i, l):
+    components_get = components.get  # see _lookup above
     if i < l:
         for spec in reversed(specs[i].__sro__):
-            comps = components.get(spec)
+            comps = components_get(spec)
             if comps:
                 _lookupAll(comps, specs, provided, result, i+1, l)
     else:
         for iface in reversed(provided):
-            comps = components.get(iface)
+            comps = components_get(iface)
             if comps:
                 result.update(comps)
 
 def _subscriptions(components, specs, provided, name, result, i, l):
+    components_get = components.get  # see _lookup above
     if i < l:
         for spec in reversed(specs[i].__sro__):
-            comps = components.get(spec)
+            comps = components_get(spec)
             if comps:
                 _subscriptions(comps, specs, provided, name, result, i+1, l)
     else:
         for iface in reversed(provided):
-            comps = components.get(iface)
+            comps = components_get(iface)
             if comps:
                 comps = comps.get(name)
                 if comps:
